# Code for Bayesian parameter estimation
# Inference of intensity based models  for load-sharing systems with damage accumulation
# Renate Meyer

library(copula)
library(rjags)
library(xtable)
library('coda')

source("Beam_data.asc")

n<-10
y<-matrix(0,nrow=n,ncol=21) # jumptimes
t<-matrix(0,nrow=n,ncol=21) # gaptimes, W in manuscript
a<-matrix(0,nrow=n,ncol=21) #time-dependent, piecewise constant covariate
alog<-matrix(0,nrow=n,ncol=21) #time-dependent, piecewise constant covariate
A<-matrix(0,nrow=n,ncol=21) #time-dependent, damage accumulation
C<-matrix(0,nrow=n,ncol=21) #
ni<-rep(0,n)
id<-rep(0,n)
delta<-rep(1,n) # 0 means censored observation 
tauj<-rep(0,n)   # max breaking time

ni<-c(15,9,12,7,4,17,18,18,20,19,1,17) 
for(i in 1:n){
  y[i,1:(ni[i])] <- jumptimes[[i]]
  tauj[i]<-max(y[i,1:(ni[i])])
  t[i,1]<-y[i,1]
  for (j in 2:(ni[i])){
    t[i,j] <- y[i,j]-y[i,j-1]
  }
}
delta[4] <- 0
delta[5] <- 0
delta[9] <- 0
k <- ni[1:n]
ni[4]<-ni[4]+1
ni[5]<-ni[5]+1
ni[9]<-ni[9]+1
t[4,ni[4]] <- censorTimes[4]- y[4,(ni[4]-1)]
t[5,ni[5]] <- censorTimes[5]- y[5,(ni[5]-1)]
t[9,ni[9]] <- censorTimes[9]- y[9,(ni[9]-1)]
t[10,18]<-10 # to avoid zeros
t[8,4]<-10   # to avoid zeros
#k <- ni[1:n]
tau<-mean(tauj)

#define first covariate (load sharing stress, log(s) in manuscript)
for(i in 1:n){
  for(j in 1:ni[i]){
  a[i,j]<-  ss[i]/(35-(j-1))
  alog[i,j]<-log(a[i,j])
  }}

#define 2. covariate (stress accumulation, C in manuscript)
for(i in 1:n){
  for(j in 1:ni[i]){
    C[i,j]<-sum(a[i,1:j]*t[i,1:j])
  }}

# Model without damage accumulation, same as Exponential waiting times
BUGSmodel = "model
{
theta1~ dnorm(0,0.001)
theta2~dgamma(0.001,0.001)

for (i in 1:n){
  for (j in 1:ni[i]){
      llambda[i,j] <--theta1-log(tau)+theta2*alog[i,j]
      lambda[i,j]<-exp(llambda[i,j])
      }
				
 # likelihood specification using zeros' trick
			
  zeros[i] ~ dpois(mu[i])

  l[i] <- sum(llambda[i,1:k[i]]) - sum(lambda[i,1:ni[i]]*t[i,1:ni[i]])
  mu[i]<- - l[i]
} 
 llike<-sum(l[1:n])
}
"


#fit model W-0 without damage accumulation
data=list(n=n,k=k,ni=ni, alog=alog,t=t,zeros=rep(0,n),tau=tau)
inits=list(list(theta1=1, theta2=1),list(theta1=2, theta2=2))
parameters=c("theta1","theta2","llike")
burn_in = 10000
steps = 20000
thin = 1
seed = 42


foo <- jags.model(textConnection(BUGSmodel),data=data, inits=inits,,n.chains=2) 
update(foo, burn_in)
out1 <- coda.samples(model=foo, variable.names=parameters, n.iter=steps, thin=thin)
save(out1,file="without.RData")
summary(out1)
dic.samples(model=foo, n.iter=steps, type="pD")

plot(out1)
geweke.diag(out1)
gelman.diag(out1)
raftery.diag(out1)
heidel.diag(out1)
autocorr.diag(out1)
crosscorr(out1)



###########################################################################################


# Model with multiplicative damage accumulation, 


BUGSmodelmult = "model
{
  theta1~ dnorm(0,0.001)
  theta2~dgamma(0.001,0.001)
  theta3~dgamma(0.001,0.001)

  
  for (i in 1:n){
  for (j in 1:ni[i]){
  llambda[i,j] <--theta1-log(tau)+theta2*alog[i,j] + theta3*log(C[i,j]/tau)
  lambda[i,j]<-exp(llambda[i,j])
  }
  
  # likelihood specification using zeros' trick
  
  zeros[i] ~ dpois(mu[i])
  
  l[i] <- sum(llambda[i,1:k[i]]) - 
         exp(-theta1)/(theta3+1)/pow(tau,theta3+1)*
         ( pow(a[i,1],theta2-1)*pow(C[i,1],theta3+1) +
         sum( pow(a[i,2:ni[i]],theta2-1)*(pow(C[i,2:ni[i]],theta3+1)-pow(C[i,1:(ni[i]-1)],theta3+1) ) ) )
 
  mu[i]<- - l[i]
  } 
  llike<-sum(l[1:n])
  }
  "
  
  
  #fit model W-0 with multiplicative damage accumulation
  data=list(n=n,k=k,ni=ni, a=a, alog=alog,zeros=rep(0,n),tau=tau,C=C)
  inits=list(list(theta1=1, theta2=1, theta3=1),list(theta1=2, theta2=2,theta3=0.5))
  parameters=c("theta1","theta2","theta3","llike")
  burn_in = 10000
  steps = 40000
  thin = 2
  seed = 42
  
  
  foo <- jags.model(textConnection(BUGSmodelmult),data=data, inits=inits,,n.chains=2) 
  update(foo, burn_in)
  out2 <- coda.samples(model=foo, variable.names=parameters, n.iter=steps, thin=thin)
  save(out2,file="multiplicative.RData")
  summary(out2)
  dic.samples(model=foo, n.iter=steps, type="pD")
  
  plot(out2)
  geweke.diag(out2)
  gelman.diag(out2)
  raftery.diag(out2)
  heidel.diag(out2)
  autocorr.diag(out2)
  crosscorr(out2)
  
  
  ###########################################################################################
  
  
  # Model with additive damage accumulation, 
  
  
  BUGSmodeladd = "model
  {
  theta1~ dnorm(0,0.01)
  theta2~dgamma(0.01,0.01)
  theta3~dgamma(0.01,0.01)
  
  for (i in 1:n){
  for (j in 1:ni[i]){
  llambda[i,j] <- -theta1-log(tau)+theta2*log(a[i,j] + theta3*C[i,j]/tau)
  lambda[i,j]<-exp(llambda[i,j])
  }
  
  # likelihood specification using zeros' trick
  
  zeros[i] ~ dpois(mu[i])
  
  l[i] <- sum(llambda[i,1:k[i]]) - 
  exp(-theta1)/(theta2+1)/theta3*
   (pow(a[i,1]+theta3*C[i,1]/tau,theta2+1)/a[i,1] -pow(a[i,1],theta2) 
 + sum((pow(a[i,2:ni[i]]+theta3*C[i,2:ni[i]]/tau,theta2+1) -pow(a[i,2:ni[i]]+ theta3*C[i,1:(ni[i]-1)]/tau,theta2+1))/a[i,2:ni[i]]))
  
  
  #mu[i]<- - l[i] +10
  mu[i]<- - l[i]
  } 
  llike<-sum(l[1:n])
  }
  "
  
  
  #fit model W-0 with additive damage accumulation
  data=list(n=n,k=k,ni=ni, a=a, zeros=rep(0,n),tau=tau,C=C)
  inits=list(list(theta1=2.039, theta2=3.462, theta3=0.186,
                  .RNG.name="base::Super-Duper", .RNG.seed=1))#,list(theta1=2, theta2=3,theta3=0.2))
  parameters=c("theta1","theta2","theta3","llike")
  burn_in = 10000
  steps = 17000
  thin = 2
  seed = 42
  
  
  foo <- jags.model(textConnection(BUGSmodeladd),data=data, inits=inits,n.chains=1) 
  update(foo, burn_in)
  out3 <- coda.samples(model=foo, variable.names=parameters, n.iter=steps, thin=thin)
  save(out3,file="additive.RData")
  summary(out3)
  dic.samples(model=foo, n.iter=steps, type="pD")
  
  plot(out3)
  geweke.diag(out3)
  gelman.diag(out3)
  raftery.diag(out3)
  heidel.diag(out3)
  autocorr.diag(out3)
  crosscorr(out3)







